@@ -3,6 +3,7 @@ pub mod client;
 pub mod context;
 pub mod csml_bot;
 pub mod csml_flow;
+pub mod csml_logs;
 pub mod csml_result;
 pub mod data;
 pub mod error_info;
@@ -22,7 +23,7 @@ pub mod warnings;
 pub use ast::Interval;
 pub use client::Client;
 pub use context::{ApiInfo, Context};
-pub use csml_bot::CsmlBot;
+pub use csml_bot::{CsmlBot, Module};
 pub use csml_flow::CsmlFlow;
 pub use csml_result::CsmlResult;
 pub use data::Data;
@@ -35,7 +36,7 @@ pub use message::Message;
 pub use message_data::MessageData;
 pub use position::Position;
 
-pub use msg::{MSG};
+pub use msg::MSG;
 
 // limit of steps in a single execution
-pub static STEP_LIMIT: i32 = 100;
\ No newline at end of file
+pub static STEP_LIMIT: i32 = 100;
