@@ -1,11 +1,11 @@
 use crate::data::position::Position;
-use crate::data::primitive::{Primitive, PrimitiveString, PrimitiveObject};
-use crate::data::{Interval, Data};
+use crate::data::primitive::{Primitive, PrimitiveObject, PrimitiveString};
+use crate::data::{Data, Interval};
 use crate::error_format::*;
 
 use std::cmp::Ordering;
-use std::ops::Add;
 use std::collections::HashMap;
+use std::ops::Add;
 
 ////////////////////////////////////////////////////////////////////////////////
 // DATA STRUCTURES
@@ -19,6 +19,7 @@ pub struct Literal {
     pub primitive: Box<dyn Primitive>,
     // this adds complementary information about the origin of the variable
     pub additional_info: Option<HashMap<String, Literal>>,
+    pub secure_variable: bool,
     pub interval: Interval,
 }
 
@@ -48,7 +49,6 @@ pub fn get_info(
     let usage = "get_info(Optional<String: search_key>) => Literal";
 
     match (additional_info, args.get("arg0")) {
-
         (Some(map), None) => Ok(PrimitiveObject::get_literal(map, interval)),
 
         (Some(map), Some(key)) => {
@@ -71,9 +71,9 @@ pub fn get_info(
                     Ok(lit)
                 }
             }
-        },
+        }
 
-        _ => Ok(PrimitiveString::get_literal("Null", interval))
+        _ => Ok(PrimitiveString::get_literal("Null", interval)),
     }
 }
 
@@ -82,7 +82,7 @@ pub fn create_error_info(error_msg: &str, interval: Interval) -> HashMap<String,
 
     map.insert(
         "error".to_owned(),
-        PrimitiveString::get_literal(error_msg, interval)
+        PrimitiveString::get_literal(error_msg, interval),
     );
 
     map
@@ -134,19 +134,32 @@ impl Literal {
             }
             None => {
                 let mut info = HashMap::new();
-                info.insert(key.to_owned(),value);
+                info.insert(key.to_owned(), value);
 
                 self.additional_info = Some(info);
             }
         }
     }
 
+    pub fn add_info_block(&mut self, info: HashMap<String, Literal>) {
+        match self.additional_info {
+            Some(ref mut map) => {
+                for (key, value) in info {
+                    map.insert(key, value);
+                }
+            }
+            None => {
+                self.additional_info = Some(info);
+            }
+        }
+    }
+
     pub fn add_error_to_info(&mut self, error_msg: &str) {
         match self.additional_info {
             Some(ref mut map) => {
                 map.insert(
                     "error".to_owned(),
-                    PrimitiveString::get_literal(error_msg, self.interval)
+                    PrimitiveString::get_literal(error_msg, self.interval),
                 );
             }
             None => {
@@ -158,10 +171,12 @@ impl Literal {
 
     pub fn add_literal_to_info(&mut self, key: String, lit: Literal) {
         match self.additional_info {
-            Some(ref mut map) => {map.insert(key,lit);},
+            Some(ref mut map) => {
+                map.insert(key, lit);
+            }
             None => {
                 let mut map = HashMap::new();
-                map.insert(key,lit);
+                map.insert(key, lit);
 
                 self.additional_info = Some(map);
             }
