@@ -24,7 +24,7 @@ pub use object::PrimitiveObject;
 pub use string::PrimitiveString;
 
 use crate::data::primitive::tools::*;
-use crate::data::{Data, Interval, Literal, Message, MessageData, MSG};
+use crate::data::{Data, Interval, Literal, MemoryType, Message, MessageData, MSG};
 use crate::error_format::*;
 
 use std::cmp::Ordering;
@@ -78,6 +78,7 @@ pub trait Primitive: Send {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
         additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         content_type: &ContentType,
@@ -111,6 +112,7 @@ impl dyn Primitive {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
         additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         content_type: &ContentType,
@@ -121,8 +123,17 @@ impl dyn Primitive {
     ) -> Result<Literal, ErrorInfo> {
         *mem_update = false;
 
-        let (res, right) =
-            self.do_exec(name, args, additional_info, interval, content_type, data, msg_data, sender)?;
+        let (res, right) = self.do_exec(
+            name,
+            args,
+            mem_type,
+            additional_info,
+            interval,
+            content_type,
+            data,
+            msg_data,
+            sender,
+        )?;
         if right == Right::Write {
             *mem_update = true;
         }
@@ -340,7 +351,12 @@ impl Add for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(float).do_add(&PrimitiveFloat::new(rhs.value as f64))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} + {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -353,7 +369,12 @@ impl Add for Box<dyn Primitive> {
                 match get_integer(&lhs.value) {
                     Ok(Integer::Int(int)) => PrimitiveFloat::new(int as f64).do_add(rhs),
                     Ok(Integer::Float(float)) => PrimitiveFloat::new(float).do_add(rhs),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} + {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -367,7 +388,12 @@ impl Add for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(lhs.value as f64).do_add(&PrimitiveFloat::new(float))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} + {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -380,7 +406,12 @@ impl Add for Box<dyn Primitive> {
                 match get_integer(&rhs.value) {
                     Ok(Integer::Int(int)) => lhs.do_add(&PrimitiveFloat::new(int as f64)),
                     Ok(Integer::Float(float)) => lhs.do_add(&PrimitiveFloat::new(float)),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} + {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
 
@@ -431,7 +462,12 @@ impl Sub for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(float).do_sub(&PrimitiveFloat::new(rhs.value as f64))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} - {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -444,7 +480,12 @@ impl Sub for Box<dyn Primitive> {
                 match get_integer(&lhs.value) {
                     Ok(Integer::Int(int)) => PrimitiveFloat::new(int as f64).do_sub(rhs),
                     Ok(Integer::Float(float)) => PrimitiveFloat::new(float).do_sub(rhs),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} - {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -458,7 +499,12 @@ impl Sub for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(lhs.value as f64).do_sub(&PrimitiveFloat::new(float))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} - {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -471,7 +517,12 @@ impl Sub for Box<dyn Primitive> {
                 match get_integer(&rhs.value) {
                     Ok(Integer::Int(int)) => lhs.do_sub(&PrimitiveFloat::new(int as f64)),
                     Ok(Integer::Float(float)) => lhs.do_sub(&PrimitiveFloat::new(float)),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} - {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             _ => Err(format!(
@@ -521,7 +572,12 @@ impl Div for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(float).do_div(&PrimitiveFloat::new(rhs.value as f64))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} / {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -534,7 +590,12 @@ impl Div for Box<dyn Primitive> {
                 match get_integer(&lhs.value) {
                     Ok(Integer::Int(int)) => PrimitiveFloat::new(int as f64).do_div(rhs),
                     Ok(Integer::Float(float)) => PrimitiveFloat::new(float).do_div(rhs),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} / {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -548,7 +609,12 @@ impl Div for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(lhs.value as f64).do_div(&PrimitiveFloat::new(float))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} / {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -561,7 +627,12 @@ impl Div for Box<dyn Primitive> {
                 match get_integer(&rhs.value) {
                     Ok(Integer::Int(int)) => lhs.do_div(&PrimitiveFloat::new(int as f64)),
                     Ok(Integer::Float(float)) => lhs.do_div(&PrimitiveFloat::new(float)),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} / {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             _ => Err(format!(
@@ -611,7 +682,12 @@ impl Mul for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(float).do_mul(&PrimitiveFloat::new(rhs.value as f64))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} * {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -624,7 +700,12 @@ impl Mul for Box<dyn Primitive> {
                 match get_integer(&lhs.value) {
                     Ok(Integer::Int(int)) => PrimitiveFloat::new(int as f64).do_mul(rhs),
                     Ok(Integer::Float(float)) => PrimitiveFloat::new(float).do_mul(rhs),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} * {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -638,7 +719,12 @@ impl Mul for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(lhs.value as f64).do_mul(&PrimitiveFloat::new(float))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} * {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -651,7 +737,12 @@ impl Mul for Box<dyn Primitive> {
                 match get_integer(&rhs.value) {
                     Ok(Integer::Int(int)) => lhs.do_mul(&PrimitiveFloat::new(int as f64)),
                     Ok(Integer::Float(float)) => lhs.do_mul(&PrimitiveFloat::new(float)),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} * {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             _ => Err(format!(
@@ -701,7 +792,12 @@ impl Rem for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(float).do_rem(&PrimitiveFloat::new(rhs.value as f64))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} % {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -714,7 +810,12 @@ impl Rem for Box<dyn Primitive> {
                 match get_integer(&lhs.value) {
                     Ok(Integer::Int(int)) => PrimitiveFloat::new(int as f64).do_rem(rhs),
                     Ok(Integer::Float(float)) => PrimitiveFloat::new(float).do_rem(rhs),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} % {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -728,7 +829,12 @@ impl Rem for Box<dyn Primitive> {
                     Ok(Integer::Float(float)) => {
                         PrimitiveFloat::new(lhs.value as f64).do_rem(&PrimitiveFloat::new(float))
                     }
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} % {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             (lhs, rhs)
@@ -741,11 +847,16 @@ impl Rem for Box<dyn Primitive> {
                 match get_integer(&rhs.value) {
                     Ok(Integer::Int(int)) => lhs.do_rem(&PrimitiveFloat::new(int as f64)),
                     Ok(Integer::Float(float)) => lhs.do_rem(&PrimitiveFloat::new(float)),
-                    Err(err) => Err(err),
+                    Err(_) => Err(format!(
+                        "{} {:?} % {:?}",
+                        ERROR_ILLEGAL_OPERATION,
+                        self.get_type(),
+                        other.get_type()
+                    )),
                 }
             }
             _ => Err(format!(
-                "{} {:?} * {:?}",
+                "{} {:?} % {:?}",
                 ERROR_ILLEGAL_OPERATION,
                 self.get_type(),
                 other.get_type()
