@@ -1,15 +1,15 @@
 use crate::data::error_info::ErrorInfo;
-use crate::data::{literal, literal::ContentType};
 use crate::data::position::Position;
 use crate::data::primitive::boolean::PrimitiveBoolean;
 use crate::data::primitive::string::PrimitiveString;
+use crate::data::{literal, literal::ContentType};
 
 use crate::data::primitive::Right;
 use crate::data::primitive::{Primitive, PrimitiveType};
 use crate::data::{
     ast::{Expr, Interval},
     message::Message,
-    Data, Literal, MessageData, MSG,
+    Data, Literal, MemoryType, MessageData, MSG,
 };
 use crate::error_format::*;
 use phf::phf_map;
@@ -165,7 +165,7 @@ impl PrimitiveClosure {
             Some(map) if map.contains_key("error") => {
                 Ok(PrimitiveBoolean::get_literal(true, interval))
             }
-            _ => Ok(PrimitiveBoolean::get_literal(false, interval))
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
         }
     }
 
@@ -218,6 +218,7 @@ impl PrimitiveClosure {
             content_type: "closure".to_owned(),
             primitive,
             additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -339,6 +340,7 @@ impl Primitive for PrimitiveClosure {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
         additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         _content_type: &ContentType,
@@ -347,9 +349,16 @@ impl Primitive for PrimitiveClosure {
         _sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<(Literal, Right), ErrorInfo> {
         if let Some((f, right)) = FUNCTIONS.get(name) {
-            let res = f(self, args, additional_info, data, interval)?;
+            if *mem_type == MemoryType::Constant && *right == Right::Write {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("{}" , ERROR_CONSTANT_MUTABLE_FUNCTION),
+                ));
+            } else {
+                let res = f(self, args, additional_info, data, interval)?;
 
-            return Ok((res, *right));
+                return Ok((res, *right));
+            }
         }
 
         Err(gen_error_info(
