@@ -1,12 +1,12 @@
 use crate::data::error_info::ErrorInfo;
+use crate::data::literal;
 use crate::data::position::Position;
 use crate::data::primitive::{
     boolean::PrimitiveBoolean, object::PrimitiveObject, string::PrimitiveString, Primitive,
     PrimitiveType, Right,
 };
-use crate::data::{literal};
 use crate::data::{
-    ast::Interval, literal::ContentType, message::Message, tokens::NULL, Data, Literal,
+    ast::Interval, literal::ContentType, message::Message, tokens::NULL, Data, Literal, MemoryType,
     MessageData, MSG,
 };
 use crate::error_format::*;
@@ -143,7 +143,7 @@ impl PrimitiveNull {
             Some(map) if map.contains_key("error") => {
                 Ok(PrimitiveBoolean::get_literal(true, interval))
             }
-            _ => Ok(PrimitiveBoolean::get_literal(false, interval))
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
         }
     }
 
@@ -185,6 +185,7 @@ impl PrimitiveNull {
             content_type: "null".to_owned(),
             primitive,
             additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -298,6 +299,7 @@ impl Primitive for PrimitiveNull {
                 content_type: "text".to_owned(),
                 primitive: Box::new(PrimitiveNull::default()),
                 additional_info: None,
+                secure_variable: false,
                 interval: Interval {
                     start_column: 0,
                     start_line: 0,
@@ -330,6 +332,7 @@ impl Primitive for PrimitiveNull {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
         additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         _content_type: &ContentType,
@@ -338,9 +341,16 @@ impl Primitive for PrimitiveNull {
         _sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<(Literal, Right), ErrorInfo> {
         if let Some((f, right)) = FUNCTIONS.get(name) {
-            let res = f(self, args, additional_info, data, interval)?;
+            if *mem_type == MemoryType::Constant && *right == Right::Write {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("{}" , ERROR_CONSTANT_MUTABLE_FUNCTION),
+                ));
+            } else {
+                let res = f(self, args, additional_info, data, interval)?;
 
-            return Ok((res, *right));
+                return Ok((res, *right));
+            }
         }
 
         Err(gen_error_info(
