@@ -16,7 +16,7 @@ use crate::data::{
 };
 use crate::error_format::*;
 use crate::interpreter::{
-    builtins::http::http_request, json_to_rust::json_to_literal,
+    builtins::http_builtin::http_request, json_to_rust::json_to_literal,
     variable_handler::match_literals::match_obj,
 };
 use std::cmp::Ordering;
@@ -119,6 +119,7 @@ const FUNCTIONS_READ: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map
 const FUNCTIONS_WRITE: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "clear_values" => (PrimitiveObject::clear_values as PrimitiveMethod, Right::Write),
     "insert" => (PrimitiveObject::insert as PrimitiveMethod, Right::Write),
+    "assign" => (PrimitiveObject::assign as PrimitiveMethod, Right::Write),
     "remove" => (PrimitiveObject::remove as PrimitiveMethod, Right::Write),
 };
 
@@ -2414,6 +2415,47 @@ impl PrimitiveObject {
         Ok(PrimitiveNull::get_literal(interval))
     }
 
+    fn assign(
+        object: &mut PrimitiveObject,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        data: &mut Data,
+        interval: Interval,
+        _content_type: &str,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "assign(obj: Object) => null";
+
+        if args.len() != 1 {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let obj = match args.get("arg0") {
+            Some(res) if res.primitive.get_type() == PrimitiveType::PrimitiveObject => {
+                Literal::get_value::<HashMap<String, Literal>>(
+                    &res.primitive,
+                    &data.context.flow,
+                    interval,
+                    ERROR_OBJECT_ASSIGN.to_owned(),
+                )?
+            }
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    ERROR_OBJECT_ASSIGN.to_owned(),
+                ));
+            }
+        };
+
+        object
+            .value
+            .extend(obj.into_iter().map(|(k, v)| (k.clone(), v.clone())));
+
+        Ok(PrimitiveNull::get_literal(interval))
+    }
+
     fn remove(
         object: &mut PrimitiveObject,
         args: &HashMap<String, Literal>,
