@@ -17,6 +17,12 @@ pub struct Warnings {
     pub position: Position,
 }
 
+#[derive(Debug, Clone)]
+pub enum DisplayWarnings {
+    On,
+    Off,
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // PRIVATE FUNCTION
 ////////////////////////////////////////////////////////////////////////////////
