@@ -1,4 +1,5 @@
 pub mod expr_to_literal;
+pub mod forget_memories;
 pub mod gen_generic_component;
 pub mod gen_literal;
 pub mod interval;
@@ -6,7 +7,6 @@ pub mod match_literals;
 pub mod memory;
 pub mod operations;
 pub mod resolve_csml_object;
-pub mod forget_memories;
 
 use crate::data::literal::ContentType;
 pub use expr_to_literal::{expr_to_literal, resolve_fn_args};
@@ -19,7 +19,8 @@ use crate::data::primitive::{
 use crate::data::{
     ast::{Expr, Function, GotoValueType, Identifier, Interval, PathLiteral, PathState},
     data::Data,
-    tokens::{COMPONENT, EVENT, _ENV, _METADATA, _MEMORY},
+    tokens::{COMPONENT, EVENT, _ENV, _MEMORY, _METADATA},
+    warnings::DisplayWarnings,
     ArgsType, Literal, MemoryType, MessageData, MSG,
 };
 use crate::error_format::*;
@@ -48,9 +49,23 @@ fn get_var_from_step_var<'a>(
     }
 }
 
+fn get_var_from_constant<'a>(
+    name: &Identifier,
+    data: &'a mut Data,
+) -> Result<&'a mut Literal, ErrorInfo> {
+    match data.constants.get_mut(&name.ident) {
+        Some(lit) => Ok(lit),
+        None => Err(gen_error_info(
+            Position::new(name.interval, &data.context.flow),
+            format!("< {} > {}", name.ident, ERROR_STEP_MEMORY),
+        )),
+    }
+}
+
 fn loop_path(
     mut lit: &mut Literal,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
+    mem_type: &MemoryType,
     new: Option<Literal>,
     path: &mut Iter<(Interval, PathLiteral)>,
     content_type: &ContentType,
@@ -78,24 +93,28 @@ fn loop_path(
                             Position::new(*interval, &data.context.flow),
                             format!("[{}] {}", index, ERROR_ARRAY_INDEX),
                         );
-                        let null = match condition {
-                            true => PrimitiveNull::get_literal(err.position.interval),
-                            false => MSG::send_error_msg(&sender, msg_data, Err(err)),
+                        let null = match dis_warnings {
+                            &DisplayWarnings::Off => {
+                                PrimitiveNull::get_literal(err.position.interval)
+                            }
+                            &DisplayWarnings::On => {
+                                MSG::send_error_msg(&sender, msg_data, Err(err))
+                            }
                         };
                         return Ok((null, tmp_update_var));
                     }
                 }
             }
-            PathLiteral::VecIndex(index) => match get_at_index(lit, &data.context.flow,*index) {
+            PathLiteral::VecIndex(index) => match get_at_index(lit, &data.context.flow, *index) {
                 Some(new_lit) => lit = new_lit,
                 None => {
                     let err = gen_error_info(
                         Position::new(*interval, &data.context.flow),
                         format!("[{}] {}", index, ERROR_ARRAY_INDEX),
                     );
-                    let null = match condition {
-                        true => PrimitiveNull::get_literal(err.position.interval),
-                        false => MSG::send_error_msg(&sender, msg_data, Err(err)),
+                    let null = match dis_warnings {
+                        &DisplayWarnings::Off => PrimitiveNull::get_literal(err.position.interval),
+                        &DisplayWarnings::On => MSG::send_error_msg(&sender, msg_data, Err(err)),
                     };
                     return Ok((null, tmp_update_var));
                 }
@@ -113,6 +132,7 @@ fn loop_path(
                     lit.primitive.exec(
                         "insert",
                         &args,
+                        mem_type,
                         &lit.additional_info,
                         interval.to_owned(),
                         content_type,
@@ -130,10 +150,23 @@ fn loop_path(
                                 Position::new(*interval, &data.context.flow),
                                 format!("[{}] {}", key, ERROR_OBJECT_GET),
                             );
-                            let null = match condition {
-                                true => PrimitiveNull::get_literal(err.position.interval),
-                                false => MSG::send_error_msg(&sender, msg_data, Err(err)),
+
+                            let error =
+                                PrimitiveString::get_literal(&err.message, err.position.interval);
+
+                            // if value does not exist in memory we create a null value and we apply all the path actions
+                            // if we are not in a condition an error message is created and send
+                            let mut null = match dis_warnings {
+                                &DisplayWarnings::Off => {
+                                    PrimitiveNull::get_literal(err.position.interval)
+                                }
+                                &DisplayWarnings::On => {
+                                    MSG::send_error_msg(&sender, msg_data, Err(err))
+                                }
                             };
+
+                            null.add_info("error", error);
+
                             return Ok((null, tmp_update_var));
                         }
                     }
@@ -165,6 +198,7 @@ fn loop_path(
                 let mut return_lit = match lit.primitive.exec(
                     name,
                     args,
+                    mem_type,
                     &lit.additional_info,
                     *interval,
                     content_type,
@@ -180,7 +214,8 @@ fn loop_path(
                 let content_type = ContentType::get(&return_lit);
                 let (lit_new, ..) = loop_path(
                     &mut return_lit,
-                    false,
+                    &DisplayWarnings::On,
+                    mem_type,
                     None,
                     path,
                     &content_type,
@@ -244,7 +279,6 @@ fn loop_path(
 // PUBLIC FUNCTION
 ////////////////////////////////////////////////////////////////////////////////
 
-//TODO: return Warning or Error Component
 pub fn get_literal<'a, 'b>(
     literal: &'a mut Literal,
     index: Option<Literal>,
@@ -286,7 +320,11 @@ pub fn get_literal<'a, 'b>(
     }
 }
 
-pub fn get_string_index(lit: Literal, flow_name: &str, index: usize) -> Result<Option<Literal>, ErrorInfo> {
+pub fn get_string_index(
+    lit: Literal,
+    flow_name: &str,
+    index: usize,
+) -> Result<Option<Literal>, ErrorInfo> {
     let array = get_array(lit, flow_name, ERROR_INDEXING.to_owned())?;
 
     match array.get(index) {
@@ -295,7 +333,11 @@ pub fn get_string_index(lit: Literal, flow_name: &str, index: usize) -> Result<O
     }
 }
 
-pub fn get_at_index<'a>(lit: &'a mut Literal, flow_name: &str, index: usize) -> Option<&'a mut Literal> {
+pub fn get_at_index<'a>(
+    lit: &'a mut Literal,
+    flow_name: &str,
+    index: usize,
+) -> Option<&'a mut Literal> {
     let vec = Literal::get_mut_value::<Vec<Literal>>(
         &mut lit.primitive,
         flow_name,
@@ -306,7 +348,11 @@ pub fn get_at_index<'a>(lit: &'a mut Literal, flow_name: &str, index: usize) ->
     vec.get_mut(index)
 }
 
-pub fn get_value_from_key<'a>(lit: &'a mut Literal, flow_name: &str, key: &str) -> Option<&'a mut Literal> {
+pub fn get_value_from_key<'a>(
+    lit: &'a mut Literal,
+    flow_name: &str,
+    key: &str,
+) -> Option<&'a mut Literal> {
     let map = Literal::get_mut_value::<HashMap<String, Literal>>(
         &mut lit.primitive,
         flow_name,
@@ -319,7 +365,7 @@ pub fn get_value_from_key<'a>(lit: &'a mut Literal, flow_name: &str, key: &str)
 
 pub fn resolve_path(
     path: &[(Interval, PathState)],
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     data: &mut Data,
     msg_data: &mut MessageData,
     sender: &Option<mpsc::Sender<MSG>>,
@@ -329,7 +375,7 @@ pub fn resolve_path(
     for (interval, node) in path.iter() {
         match node {
             PathState::ExprIndex(expr) => {
-                let lit = expr_to_literal(&expr, condition, None, data, msg_data, sender)?;
+                let lit = expr_to_literal(&expr, dis_warnings, None, data, msg_data, sender)?;
                 if let Ok(val) = Literal::get_value::<i64>(
                     &lit.primitive,
                     &data.context.flow,
@@ -360,7 +406,7 @@ pub fn resolve_path(
                 PathLiteral::Func {
                     name: name.to_owned(),
                     interval: interval.to_owned(),
-                    args: resolve_fn_args(&args, data, msg_data, sender)?,
+                    args: resolve_fn_args(&args, data, msg_data, dis_warnings, sender)?,
                 },
             )),
             PathState::StringIndex(key) => {
@@ -371,10 +417,10 @@ pub fn resolve_path(
     Ok(new_path)
 }
 
-//TODO: Add Warning for nonexisting key
 pub fn exec_path_actions(
     lit: &mut Literal,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
+    mem_type: &MemoryType,
     new: Option<Literal>,
     path: &Option<Vec<(Interval, PathLiteral)>>,
     content_type: &ContentType,
@@ -386,7 +432,8 @@ pub fn exec_path_actions(
         let mut path = vec.iter();
         let (return_lit, update) = loop_path(
             lit,
-            condition,
+            dis_warnings,
+            mem_type,
             new,
             &mut path,
             content_type,
@@ -406,36 +453,33 @@ pub fn exec_path_actions(
     }
 }
 
-
 fn get_flow_context(data: &mut Data, interval: Interval) -> HashMap<String, Literal> {
-
     let mut flow_context = HashMap::new();
 
     flow_context.insert(
         "current_step".to_owned(),
-        PrimitiveString::get_literal(&data.context.step, interval)
+        PrimitiveString::get_literal(&data.context.step, interval),
     );
     flow_context.insert(
         "current_flow".to_owned(),
-        PrimitiveString::get_literal(&data.context.flow, interval)
+        PrimitiveString::get_literal(&data.context.flow, interval),
     );
 
     flow_context.insert(
         "default_flow".to_owned(),
-        PrimitiveString::get_literal(&data.context.flow, interval)
+        PrimitiveString::get_literal(&data.context.flow, interval),
     );
 
     if let Some(previous_info) = &data.previous_info {
         flow_context.insert(
             "previous_step".to_owned(),
-            PrimitiveString::get_literal(&previous_info.step_at_flow.0, interval)
+            PrimitiveString::get_literal(&previous_info.step_at_flow.0, interval),
         );
         flow_context.insert(
             "previous_flow".to_owned(),
-            PrimitiveString::get_literal(&previous_info.step_at_flow.1, interval)
+            PrimitiveString::get_literal(&previous_info.step_at_flow.1, interval),
         );
     }
-    
 
     flow_context
 }
@@ -443,7 +487,7 @@ fn get_flow_context(data: &mut Data, interval: Interval) -> HashMap<String, Lite
 pub fn get_metadata_context_literal(
     path: &[(Interval, PathLiteral)],
     interval: Interval,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     data: &mut Data,
     msg_data: &mut MessageData,
     sender: &Option<mpsc::Sender<MSG>>,
@@ -459,17 +503,18 @@ pub fn get_metadata_context_literal(
                     path_skip += 1;
 
                     lit.to_owned()
-                },
+                }
                 None => PrimitiveObject::get_literal(&flow_context, *interval),
             }
-        },
-        _ => PrimitiveObject::get_literal(&flow_context, interval)
+        }
+        _ => PrimitiveObject::get_literal(&flow_context, interval),
     };
 
     let content_type = ContentType::get(&lit);
     let (lit, _tmp_mem_update) = exec_path_actions(
         &mut lit,
-        condition,
+        dis_warnings,
+        &MemoryType::Metadata,
         None,
         &Some(path[path_skip..].to_owned()),
         &content_type,
@@ -480,17 +525,32 @@ pub fn get_metadata_context_literal(
     Ok(lit)
 }
 
+// pub enum MemoryType {
+//     Event(String),
+//     Metadata,
+//     Use,
+//     Remember,
+//     Constant,
+// }
+
 pub fn get_literal_from_metadata(
     path: &[(Interval, PathLiteral)],
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     data: &mut Data,
     msg_data: &mut MessageData,
     sender: &Option<mpsc::Sender<MSG>>,
 ) -> Result<Literal, ErrorInfo> {
     let mut lit = match path.get(0) {
         Some((interval, PathLiteral::MapIndex(name))) if name == "_context" => {
-            return get_metadata_context_literal(path, *interval, condition, data, msg_data, sender);
-        },
+            return get_metadata_context_literal(
+                path,
+                *interval,
+                dis_warnings,
+                data,
+                msg_data,
+                sender,
+            );
+        }
         Some((interval, PathLiteral::MapIndex(name))) => match data.context.metadata.get(name) {
             Some(lit) => lit.to_owned(),
             None => PrimitiveNull::get_literal(interval.to_owned()),
@@ -507,7 +567,8 @@ pub fn get_literal_from_metadata(
     let content_type = ContentType::get(&lit);
     let (lit, _tmp_mem_update) = exec_path_actions(
         &mut lit,
-        condition,
+        dis_warnings,
+        &MemoryType::Metadata,
         None,
         &Some(path[1..].to_owned()),
         &content_type,
@@ -520,7 +581,7 @@ pub fn get_literal_from_metadata(
 
 pub fn get_var(
     var: Identifier,
-    condition: bool, // TODO: find better method than this
+    dis_warnings: &DisplayWarnings,
     path: Option<&[(Interval, PathState)]>,
     data: &mut Data,
     msg_data: &mut MessageData,
@@ -533,16 +594,17 @@ pub fn get_var(
             gen_literal_from_component(*interval, path, data, msg_data, sender)
         }
         name if name == EVENT => {
-            gen_literal_from_event(*interval, condition, path, data, msg_data, sender)
+            gen_literal_from_event(*interval, dis_warnings, path, data, msg_data, sender)
         }
         name if name == _ENV => match path {
             Some(path) => {
-                let path = resolve_path(path, condition, data, msg_data, sender)?;
+                let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
 
                 let content_type = ContentType::get(&data.env);
                 let (lit, _tmp_mem_update) = exec_path_actions(
                     &mut data.env.clone(),
-                    condition,
+                    dis_warnings,
+                    &MemoryType::Constant,
                     None,
                     &Some(path.to_owned()),
                     &content_type,
@@ -556,8 +618,8 @@ pub fn get_var(
         },
         name if name == _METADATA => match path {
             Some(path) => {
-                let path = resolve_path(path, condition, data, msg_data, sender)?;
-                get_literal_from_metadata(&path, condition, data, msg_data, sender)
+                let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
+                get_literal_from_metadata(&path, dis_warnings, data, msg_data, sender)
             }
             None => Ok(PrimitiveObject::get_literal(
                 &data.context.metadata,
@@ -570,10 +632,11 @@ pub fn get_var(
 
             match path {
                 Some(path) => {
-                    let path = resolve_path(path, condition, data, msg_data, sender)?;
+                    let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
                     let (lit, _tmp_mem_update) = exec_path_actions(
                         &mut lit,
-                        condition,
+                        dis_warnings,
+                        &MemoryType::Remember,
                         None,
                         &Some(path),
                         &ContentType::Primitive,
@@ -584,9 +647,9 @@ pub fn get_var(
 
                     Ok(lit)
                 }
-                None => Ok(lit)
+                None => Ok(lit),
             }
-        },
+        }
         _ => {
             // ######################
             //// TODO:
@@ -594,12 +657,13 @@ pub fn get_var(
             // in the future we need to refacto this code to avoid any scope copy like this
             let (
                 tmp_flows,
+                tmp_extern_flows,
                 tmp_flow,
                 tmp_default_flow,
                 mut tmp_context,
                 tmp_event,
                 tmp_env,
-                tmp_loop_indexs,
+                tmp_loop_indexes,
                 tmp_loop_index,
                 mut tmp_step_count,
                 tmp_step_vars,
@@ -609,14 +673,15 @@ pub fn get_var(
 
             let mut new_scope_data = Data::new(
                 &tmp_flows,
+                &tmp_extern_flows,
                 &tmp_flow,
                 tmp_default_flow,
                 &mut tmp_context,
                 &tmp_event,
                 &tmp_env,
-                tmp_loop_indexs,
+                tmp_loop_indexes,
                 tmp_loop_index,
-                &mut tmp_step_count, 
+                &mut tmp_step_count,
                 tmp_step_vars,
                 data.previous_info.clone(),
                 &tmp_custom_component,
@@ -624,11 +689,12 @@ pub fn get_var(
             );
             // #####################
 
-            match get_var_from_mem(var.to_owned(), condition, path, data, msg_data, sender) {
+            match get_var_from_mem(var.to_owned(), dis_warnings, path, data, msg_data, sender) {
                 Ok((lit, name, mem_type, path)) => {
                     let result = exec_path_actions(
                         lit,
-                        condition,
+                        dis_warnings,
+                        &mem_type,
                         None,
                         &path,
                         &ContentType::get(&lit),
@@ -636,6 +702,7 @@ pub fn get_var(
                         msg_data,
                         sender,
                     );
+
                     let (new_literal, update_mem) = match result {
                         Ok((lit, update)) => (lit, update),
                         Err(err) => (MSG::send_error_msg(&sender, msg_data, Err(err)), false),
@@ -653,22 +720,27 @@ pub fn get_var(
                     Ok(new_literal)
                 }
                 Err(err) => {
+                    let error = PrimitiveString::get_literal(&err.message, err.position.interval);
+
                     // if value does not exist in memory we create a null value and we apply all the path actions
                     // if we are not in a condition an error message is created and send
-                    let mut null = match condition {
-                        true => PrimitiveNull::get_literal(err.position.interval),
-                        false => MSG::send_error_msg(&sender, msg_data, Err(err)),
+                    let mut null = match dis_warnings {
+                        &DisplayWarnings::Off => PrimitiveNull::get_literal(err.position.interval),
+                        &DisplayWarnings::On => MSG::send_error_msg(&sender, msg_data, Err(err)),
                     };
 
+                    null.add_info("error", error);
+
                     let path = if let Some(p) = path {
-                        Some(resolve_path(p, condition, data, msg_data, sender)?)
+                        Some(resolve_path(p, dis_warnings, data, msg_data, sender)?)
                     } else {
                         None
                     };
                     let content_type = ContentType::get(&null);
                     let (new_literal, ..) = exec_path_actions(
                         &mut null,
-                        condition,
+                        dis_warnings,
+                        &MemoryType::Use,
                         None,
                         &path,
                         &content_type,
@@ -685,7 +757,7 @@ pub fn get_var(
 
 pub fn get_var_from_mem<'a>(
     name: Identifier,
-    condition: bool, // TODO: find better method than this
+    dis_warnings: &DisplayWarnings,
     path: Option<&[(Interval, PathState)]>,
     data: &'a mut Data,
     msg_data: &mut MessageData,
@@ -700,7 +772,7 @@ pub fn get_var_from_mem<'a>(
     ErrorInfo,
 > {
     let path = if let Some(p) = path {
-        Some(resolve_path(p, condition, data, msg_data, sender)?)
+        Some(resolve_path(p, dis_warnings, data, msg_data, sender)?)
     } else {
         None
     };
@@ -710,6 +782,10 @@ pub fn get_var_from_mem<'a>(
             let lit = get_var_from_step_var(&name, data)?;
             Ok((lit, name.ident, MemoryType::Use, path))
         }
+        var if var == "constant" => {
+            let lit = get_var_from_constant(&name, data)?;
+            Ok((lit, name.ident, MemoryType::Constant, path))
+        }
         _ => {
             let lit = search_var_memory(name.clone(), data)?;
             Ok((lit, name.ident, MemoryType::Remember, path))
@@ -728,14 +804,8 @@ pub fn search_goto_var_memory<'a>(
     match var {
         GotoValueType::Name(ident) => Ok(ident.ident.clone()),
         GotoValueType::Variable(expr) => {
-            let literal = expr_to_literal(
-                expr,
-                false,
-                None,
-                data,
-                msg_data,
-                sender,
-            )?;
+            let literal =
+                expr_to_literal(expr, &DisplayWarnings::On, None, data, msg_data, sender)?;
 
             Ok(Literal::get_value::<String>(
                 &literal.primitive,
@@ -756,11 +826,16 @@ pub fn get_string_from_complex_string(
     sender: &Option<mpsc::Sender<MSG>>,
 ) -> Result<Literal, ErrorInfo> {
     let mut new_string = String::new();
+    let mut is_secure = false;
 
-    //TODO: log error with span
     for elem in exprs.iter() {
-        match expr_to_literal(elem, false, None, data, msg_data, sender) {
-            Ok(var) => new_string.push_str(&var.primitive.to_string()),
+        match expr_to_literal(elem, &DisplayWarnings::On, None, data, msg_data, sender) {
+            Ok(var) => {
+                if var.secure_variable {
+                    is_secure = true;
+                }
+                new_string.push_str(&var.primitive.to_string())
+            }
             Err(err) => {
                 return Err(err);
             }
@@ -768,6 +843,7 @@ pub fn get_string_from_complex_string(
     }
 
     let mut result = PrimitiveString::get_literal(&new_string, interval);
+    result.secure_variable = is_secure;
     result.set_content_type("text");
 
     Ok(result)
