@@ -1,10 +1,11 @@
 pub mod data;
 pub mod error_format;
+pub mod fold_bot;
 pub mod interpreter;
 pub mod linter;
-pub mod fold_bot;
 pub mod parser;
 
+pub use data::csml_logs;
 pub use interpreter::components::load_components;
 pub use parser::step_checksum::get_step;
 
@@ -15,17 +16,19 @@ use data::ast::{Expr, Flow, InstructionScope, Interval};
 use data::context::get_hashmap_from_mem;
 use data::error_info::ErrorInfo;
 use data::event::Event;
+use data::literal::create_error_info;
 use data::message_data::MessageData;
 use data::msg::MSG;
-use data::CsmlBot;
 use data::CsmlResult;
+use data::{csml_bot::CsmlBot, CsmlFlow};
 use data::{Context, Data, Position, STEP_LIMIT};
 use error_format::*;
-use linter::{linter::lint_bot, FlowToValidate};
 use fold_bot::fold_bot as fold;
+use linter::{linter::lint_bot, FlowToValidate};
 use parser::ExitCondition;
 
 use std::collections::HashMap;
+use std::env;
 use std::sync::mpsc;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -97,7 +100,7 @@ fn execute_step(
 ////////////////////////////////////////////////////////////////////////////////
 
 pub fn get_steps_from_flow(bot: CsmlBot) -> HashMap<String, Vec<String>> {
-    let _ = env_logger::try_init();
+    csml_logs::init_logger();
 
     let mut result = HashMap::new();
 
@@ -117,9 +120,10 @@ pub fn get_steps_from_flow(bot: CsmlBot) -> HashMap<String, Vec<String>> {
 }
 
 pub fn validate_bot(bot: &CsmlBot) -> CsmlResult {
-    let _ = env_logger::try_init();
+    csml_logs::init_logger();
 
     let mut flows = vec![];
+    let mut modules = vec![];
     let mut errors = Vec::new();
     let mut imports = Vec::new();
 
@@ -145,11 +149,31 @@ pub fn validate_bot(bot: &CsmlBot) -> CsmlResult {
         }
     }
 
+    if let Some(ref mods) = bot.modules {
+        for module in mods.iter() {
+            if let Some(flow) = &module.flow {
+                match parse_flow(&flow.content, &flow.name) {
+                    Ok(ast_flow) => {
+                        modules.push(FlowToValidate {
+                            flow_name: flow.name.to_owned(),
+                            ast: ast_flow,
+                            raw_flow: &flow.content,
+                        });
+                    }
+                    Err(error) => {
+                        errors.push(error);
+                    }
+                }
+            }
+        }
+    }
+
     let mut warnings = vec![];
     // only use the linter if there is no error in the paring otherwise the linter will catch false errors
     if errors.is_empty() {
         lint_bot(
             &flows,
+            &modules,
             &mut errors,
             &mut warnings,
             &bot.native_components,
@@ -157,13 +181,19 @@ pub fn validate_bot(bot: &CsmlBot) -> CsmlResult {
         );
     }
 
-    CsmlResult::new(FlowToValidate::get_bot(flows), warnings, errors)
+    CsmlResult::new(
+        FlowToValidate::get_flows(flows),
+        FlowToValidate::get_flows(modules),
+        warnings,
+        errors,
+    )
 }
 
 pub fn fold_bot(bot: &CsmlBot) -> String {
-    let _ = env_logger::try_init();
+    csml_logs::init_logger();
 
     let mut flows = vec![];
+    let mut modules = vec![];
     let mut errors = Vec::new();
     let mut imports = Vec::new();
 
@@ -189,11 +219,31 @@ pub fn fold_bot(bot: &CsmlBot) -> String {
         }
     }
 
+    if let Some(ref mods) = bot.modules {
+        for module in mods.iter() {
+            if let Some(flow) = &module.flow {
+                match parse_flow(&flow.content, &flow.name) {
+                    Ok(ast_flow) => {
+                        modules.push(FlowToValidate {
+                            flow_name: flow.name.to_owned(),
+                            ast: ast_flow,
+                            raw_flow: &flow.content,
+                        });
+                    }
+                    Err(error) => {
+                        errors.push(error);
+                    }
+                }
+            }
+        }
+    }
+
     let mut warnings = vec![];
     // only use the fold if there is no error in the paring otherwise the linter will catch false errors
 
     fold(
         &flows,
+        &modules,
         &mut errors,
         &mut warnings,
         &bot.native_components,
@@ -201,7 +251,7 @@ pub fn fold_bot(bot: &CsmlBot) -> String {
     )
 }
 
-fn get_flows(bot: &CsmlBot) -> HashMap<String, Flow> {
+fn get_flows(bot: &CsmlBot) -> (HashMap<String, Flow>, HashMap<String, Flow>) {
     match &bot.bot_ast {
         Some(bot) => {
             let base64decoded = base64::decode(&bot).unwrap();
@@ -209,12 +259,87 @@ fn get_flows(bot: &CsmlBot) -> HashMap<String, Flow> {
         }
         None => {
             let bot = validate_bot(&bot);
-            match bot.flows {
+
+            let flows = match bot.flows {
                 Some(flows) => flows,
                 None => HashMap::new(),
+            };
+
+            let extern_flows = match bot.extern_flows {
+                Some(flows) => flows,
+                None => HashMap::new(),
+            };
+
+            (flows, extern_flows)
+        }
+    }
+}
+
+pub fn search_for_modules(bot: &mut CsmlBot) -> Result<(), String> {
+    let default_auth = env::var("MODULES_AUTH").ok();
+    let default_url = env::var("MODULES_URL").ok();
+
+    if let Some(ref mut modules) = bot.modules {
+        for module in modules.iter_mut() {
+            if module.flow.is_some() {
+                // module already downloaded
+                continue;
+            }
+
+            let request = match (&module.url, &default_url) {
+                (Some(url), _) => {
+                    let request = ureq::get(url);
+                    match &module.auth {
+                        Some(auth) => {
+                            let authorization =
+                                format!("Basic {}", base64::encode(auth.as_bytes()));
+
+                            request.set("Authorization", &authorization)
+                        }
+                        _ => request,
+                    }
+                }
+                (None, Some(url)) => {
+                    let request = ureq::get(url);
+
+                    match &default_auth {
+                        Some(auth) => {
+                            let authorization =
+                                format!("Basic {}", base64::encode(auth.as_bytes()));
+
+                            request.set("Authorization", &authorization)
+                        }
+                        _ => request,
+                    }
+                }
+                _ => {
+                    return Err(format!(
+                        "missing url in order to get module [{}]",
+                        module.name
+                    ));
+                }
+            };
+
+            match request.call() {
+                Ok(response) => {
+                    let flow_content = match response.into_string() {
+                        Ok(flow) => flow,
+                        Err(_) => return Err(format!("invalid module {}", module.name)),
+                    };
+
+                    module.flow = Some(CsmlFlow {
+                        id: module.name.clone(),
+                        name: module.name.clone(),
+                        content: flow_content,
+                        commands: vec![],
+                    });
+                }
+                Err(error) => return Err(error.to_string()),
             }
         }
     }
+
+    Ok(())
 }
 
 pub fn interpret(
@@ -223,7 +348,7 @@ pub fn interpret(
     event: Event,
     sender: Option<mpsc::Sender<MSG>>,
 ) -> MessageData {
-    let _ = env_logger::try_init();
+    csml_logs::init_logger();
 
     let mut msg_data = MessageData::default();
 
@@ -247,7 +372,7 @@ pub fn interpret(
         _ => serde_json::Map::new(),
     };
 
-    let flows = get_flows(&bot);
+    let (flows, extern_flows) = get_flows(&bot);
 
     let env = match bot.env {
         Some(env) => json_to_literal(&env, Interval::default(), &flow).unwrap(),
@@ -266,22 +391,39 @@ pub fn interpret(
         let ast = match flows.get(&flow) {
             Some(result) => result.to_owned(),
             None => {
+                let error_message = format!("flow '{}' does not exist in this bot", flow);
+                let error_info = create_error_info(&error_message, Interval::default());
+
                 return MessageData::error_to_message(
                     Err(ErrorInfo {
                         position: Position {
                             flow: flow.clone(),
                             interval: Interval::default(),
                         },
-                        message: format!("flow '{}' does not exist in this bot", flow),
+                        message: error_message,
+                        additional_info: Some(error_info),
                     }),
                     &sender,
                 );
             }
         };
 
+        // if the target flow dose not contains a 'start' flow change the target to the default_flow
+        if step == "start"
+            && ast
+                .flow_instructions
+                .get(&InstructionScope::StepScope(step.to_owned()))
+                .is_none()
+        {
+            flow = bot.default_flow.clone();
+            continue;
+        }
+
         let mut data = Data::new(
             &flows,
+            &extern_flows,
             &ast,
+            bot.default_flow.clone(),
             &mut context,
             &event,
             &env,
