@@ -9,8 +9,8 @@ use nom::{
     branch::alt,
     bytes::complete::tag,
     combinator::{map, opt},
-    error::{ErrorKind, ParseError},
-    multi::separated_list,
+    error::{ContextError, ErrorKind, ParseError},
+    multi::separated_list0,
     sequence::{preceded, terminated, tuple},
     Err, IResult,
 };
@@ -21,7 +21,7 @@ use nom::{
 
 fn parse_fn_name<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, identifier) = parse_idents_assignation(s)?;
 
@@ -30,7 +30,7 @@ where
 
 fn parse_fn_name_as_vec<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Vec<Expr>, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, expr) = parse_fn_name(s)?;
 
@@ -39,14 +39,14 @@ where
 
 fn parse_group<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Vec<Expr>, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, (vec, ..)) = preceded(
         tag(L_BRACE),
         terminated(
             tuple((
                 map(
-                    separated_list(preceded(comment, tag(COMMA)), parse_fn_name),
+                    separated_list0(preceded(comment, tag(COMMA)), parse_fn_name),
                     |vec| vec.into_iter().map(|expr| expr).collect(),
                 ),
                 opt(preceded(comment, tag(COMMA))),
@@ -60,7 +60,7 @@ where
 
 fn parse_import_params<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Vec<Expr>, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     match alt((parse_group, parse_fn_name_as_vec))(s) {
         Ok(value) => Ok(value),
@@ -72,15 +72,27 @@ where
     }
 }
 
-fn parse_from<'a, E>(s: Span<'a>) -> IResult<Span<'a>, String, E>
+fn parse_from<'a, E>(s: Span<'a>) -> IResult<Span<'a>, FromFlow, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, FROM)(s)?;
     let (s, name) = preceded(comment, get_string)(s)?;
 
-    Ok((s, name))
+    Ok((s, FromFlow::Normal(name)))
+}
+
+fn parse_from_extern_module<'a, E>(s: Span<'a>) -> IResult<Span<'a>, FromFlow, E>
+where
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
+{
+    let (s, name) = preceded(comment, get_string)(s)?;
+    let (s, ..) = get_tag(name, FROM)(s)?;
+
+    let (s, name) = preceded(comment, preceded(tag("modules/"), get_string))(s)?;
+
+    Ok((s, FromFlow::Extern(name)))
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -89,25 +101,28 @@ where
 
 pub fn parse_import_prototype<'a, E>(
     s: Span<'a>,
-) -> IResult<Span<'a>, (Interval, Vec<Expr>, Option<String>), E>
+) -> IResult<Span<'a>, (Interval, Vec<Expr>, FromFlow), E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
-    let (s, start) =  preceded(comment,get_interval)(s)?;
+    let (s, start) = preceded(comment, get_interval)(s)?;
     let (s, name) = preceded(comment, get_string)(s)?;
 
     let (s, ..) = get_tag(name, IMPORT)(s)?;
 
     let (s, fn_names) = preceded(comment, parse_import_params)(s)?;
 
-    let (s, from_flow) = opt(parse_from)(s)?;
+    let (s, from_flow) = match opt(alt((parse_from_extern_module, parse_from)))(s)? {
+        (s, Some(from_flow)) => (s, from_flow),
+        (s, None) => (s, FromFlow::None),
+    };
 
     Ok((s, (start, fn_names, from_flow)))
 }
 
 pub fn parse_import<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Vec<Instruction>, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, (interval, fn_names, from_flow)) = parse_import_prototype(s)?;
 
